@@ -14,8 +14,10 @@ class BadPhotoCopy(Augmentation):
 
     :param mask: Mask of noise to generate badphotocopy effect.
     :type mask: uint8, optional
-    :param noise_type: Types of noises to generate different mask patterns.
+    :param noise_type: Types of noises to generate different mask patterns. Use -1 to select randomly.
     :type noise_type: int, optional
+    :param noise_side: Location of noise.
+    :type noise_side: string, optional
     :param noise_iteration: Pair of ints to determine number of iterations to apply noise in the mask.
     :type noise_iteration: tuple, optional
     :param noise_size: Pair of ints to determine scale of noise in the mask.
@@ -26,12 +28,14 @@ class BadPhotoCopy(Augmentation):
     :type noise_sparsity: tuple, optional
     :param noise_concentration: Pair of floats determining concentration of noise.
     :type noise_concentration: tuple, optional
-    :param blur_noise: Flag to enable blur in noise mask.
+    :param blur_noise: Flag to enable blur in noise mask. Use -1 to select randomly.
     :type blur_noise: int, optional
     :param blur_noise_kernel: Kernel to blur noise mask.
     :type blur_noise_kernel: tuple, optional
-    :param wave_pattern: To enable wave pattern in noise.
+    :param wave_pattern: To enable wave pattern in noise. Use -1 to select randomly.
     :type wave_pattern: int, optional
+    :param edge_effect: To add sobel edge effect into the noise mask. Use -1 to select randomly.
+    :type edge_effect: int, optional
     :param p: The probability this Augmentation will be applied.
     :type p: float, optional
     """
@@ -39,21 +43,24 @@ class BadPhotoCopy(Augmentation):
     def __init__(
         self,
         mask=None,
-        noise_type=0,
-        noise_iteration=(1, 1),
-        noise_size=(1, 1),
-        noise_value=(30, 60),
-        noise_sparsity=(0.4, 0.6),
-        noise_concentration=(0.4, 0.6),
-        blur_noise=0,
+        noise_type=-1,
+        noise_side="random",
+        noise_iteration=(2, 5),
+        noise_size=(1, 3),
+        noise_value=(0, 64),
+        noise_sparsity=(0.1, 0.9),
+        noise_concentration=(0.1, 0.6),
+        blur_noise=-1,
         blur_noise_kernel=(5, 5),
-        wave_pattern=0,
+        wave_pattern=-1,
+        edge_effect=-1,
         p=1,
     ):
         """Constructor method"""
         super().__init__(p=p)
         self.mask = mask
         self.noise_type = noise_type
+        self.noise_side = noise_side
         self.noise_iteration = noise_iteration
         self.noise_size = noise_size
         self.noise_value = noise_value
@@ -62,6 +69,7 @@ class BadPhotoCopy(Augmentation):
         self.blur_noise = blur_noise
         self.blur_noise_kernel = blur_noise_kernel
         self.wave_pattern = wave_pattern
+        self.edge_effect = edge_effect
 
         # clamp values
         # noise value range from 0-255
@@ -81,7 +89,7 @@ class BadPhotoCopy(Augmentation):
 
     # Constructs a string representation of this Augmentation.
     def __repr__(self):
-        return f"BadPhotoCopy(mask={self.mask}, noise_type={self.noise_type}, noise_iteration={self.noise_iteration}, noise_size={self.noise_size}, noise_value={self.noise_value}, noise_sparsity={self.noise_sparsity}, noise_concentration={self.noise_concentration}, blur_noise={self.blur_noise}, blur_noise_kernel={self.blur_noise_kernel}, wave_pattern={self.wave_pattern}, p={self.p})"
+        return f"BadPhotoCopy(mask={self.mask}, noise_type={self.noise_type}, noise_side={self.noise_side}, noise_iteration={self.noise_iteration}, noise_size={self.noise_size}, noise_value={self.noise_value}, noise_sparsity={self.noise_sparsity}, noise_concentration={self.noise_concentration}, blur_noise={self.blur_noise}, blur_noise_kernel={self.blur_noise_kernel}, wave_pattern={self.wave_pattern}, edge_effect={self.edge_effect}, p={self.p})"
 
     def apply_wave(self, mask):
         """
@@ -89,7 +97,12 @@ class BadPhotoCopy(Augmentation):
         """
 
         # rescale mask from 0 to 255
-        mask_rescaled = mask = (((mask - np.min(mask)) / (np.max(mask) - np.min(mask))) * 255).astype("uint8")
+        # prevent zero division
+        if np.max(mask) - np.min(mask) == 0:
+            divisor = 1
+        else:
+            divisor = np.max(mask) - np.min(mask)
+        mask_rescaled = (((mask - np.min(mask)) / divisor) * 255).astype("uint8")
         mask_ysize, mask_xsize = mask_rescaled.shape
         img_wave = np.zeros_like(mask_rescaled)
 
@@ -109,15 +122,23 @@ class BadPhotoCopy(Augmentation):
         x_points.sort()
 
         # 1st point
-        points = [(0, random.randint(mask_y_one_twelve_size, mask_y_third_quarter_size))]
+        points = [
+            (0, random.randint(mask_y_one_twelve_size, mask_y_third_quarter_size)),
+        ]
         for i in range(1, number_points - 1, 1):
             # points between 1st and last point
             points.append(
-                (x_points[i - 1], random.randint(mask_y_one_twelve_size, mask_y_third_quarter_size)),
+                (
+                    x_points[i - 1],
+                    random.randint(mask_y_one_twelve_size, mask_y_third_quarter_size),
+                ),
             )  # last point
         # last point
         points.append(
-            (mask_xsize - 1, random.randint(mask_y_one_twelve_size, mask_y_third_quarter_size)),
+            (
+                mask_xsize - 1,
+                random.randint(mask_y_one_twelve_size, mask_y_third_quarter_size),
+            ),
         )  # last point
 
         # smooth points
@@ -139,7 +160,11 @@ class BadPhotoCopy(Augmentation):
         # right (noise concentrated at right edge)
         elif self.noise_type == 6:
             img_wave = np.rot90(img_wave, 3)
-            img_wave = cv2.resize(img_wave, (mask_xsize, mask_ysize), interpolation=cv2.INTER_AREA)
+            img_wave = cv2.resize(
+                img_wave,
+                (mask_xsize, mask_ysize),
+                interpolation=cv2.INTER_AREA,
+            )
             mask = img_wave * mask
         # bottom (noise concentrated at bottom edge)
         elif self.noise_type == 8:
@@ -148,11 +173,19 @@ class BadPhotoCopy(Augmentation):
         # left (noise concentrated at left edge)
         elif self.noise_type == 5:
             img_wave = np.rot90(img_wave, 1)
-            img_wave = cv2.resize(img_wave, (mask_xsize, mask_ysize), interpolation=cv2.INTER_AREA)
+            img_wave = cv2.resize(
+                img_wave,
+                (mask_xsize, mask_ysize),
+                interpolation=cv2.INTER_AREA,
+            )
             mask = img_wave * mask
         else:
             img_wave = np.rot90(img_wave, random.randint(0, 3))
-            img_wave = cv2.resize(img_wave, (mask_xsize, mask_ysize), interpolation=cv2.INTER_AREA)
+            img_wave = cv2.resize(
+                img_wave,
+                (mask_xsize, mask_ysize),
+                interpolation=cv2.INTER_AREA,
+            )
             mask = img_wave * mask
 
         # reset 0 area to white (white is background)
@@ -165,34 +198,28 @@ class BadPhotoCopy(Augmentation):
 
     def apply_augmentation(self, image):
 
+        image_shape_length = len(image.shape)
+
         # get image dimensions
-        if len(image.shape) > 2:
+        if image_shape_length > 2:
             ysize, xsize, dim = image.shape
             image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
         else:
             ysize, xsize = image.shape
 
-        # add random noise range from 0-128
-        add_edge_noise_fn = lambda x, y: random.randint(0, 128) if (y == 255 and random.random() < 0.70) else x
-        add_edge_noise = np.vectorize(add_edge_noise_fn)
-
-        image_sobel = sobel(image)
-        image_sobel = cv2.GaussianBlur(image_sobel, (3, 3), 0)
-        image_sobel[:, :][image_sobel[:, :] % 255 != 0] = 255
-        image_sobel = cv2.dilate(image_sobel, (5, 5), iterations=2)
-
-        image_sobel_sobel = sobel(image_sobel)
-        image_sobel_sobel = cv2.dilate(image_sobel_sobel, (3, 3), iterations=2)
-        image_sobel = add_edge_noise(image_sobel, image_sobel_sobel)
-        image_sobel = cv2.GaussianBlur(image_sobel, (5, 5), 0)
-
         # check if provided mask is numpy array
         if isinstance(self.mask, np.ndarray):
             mask = self.mask
+            # noise mask needs to be in grayscale form
+            if len(mask.shape) > 2:
+                mask = cv2.cvtColor(mask, cv2.COLOR_BGR2GRAY)
+
         # generate mask of noise
         else:
-
-            noise_generator = NoiseGenerator(noise_type=self.noise_type)
+            noise_generator = NoiseGenerator(
+                noise_type=self.noise_type,
+                noise_side=self.noise_side,
+            )
             mask = noise_generator.generate_noise(
                 noise_value=self.noise_value,
                 noise_iteration=self.noise_iteration,
@@ -202,7 +229,7 @@ class BadPhotoCopy(Augmentation):
             )
 
             # randomly rotate mask
-            if self.noise_type != 5 and self.noise_type != 6 and self.noise_type != 7 and self.noise_type != 8:
+            if self.noise_side == "random":
                 mask = np.rot90(mask, random.randint(0, 3))
 
         # new size after rotation
@@ -219,22 +246,30 @@ class BadPhotoCopy(Augmentation):
         mask = cv2.resize(mask, (image.shape[1], image.shape[0])).astype("uint8")
 
         # apply blur to mask of noise
-        if self.blur_noise:
+        if self.blur_noise == -1:
+            blur_noise = random.choice((0, 1))
+        else:
+            blur_noise = self.blur_noise
+        if blur_noise:
             mask = cv2.GaussianBlur(mask, self.blur_noise_kernel, 0)
 
         # apply wave pattern to mask
-        if self.wave_pattern:
+        if self.wave_pattern == -1:
+            wave_pattern = random.choice((0, 1))
+        else:
+            wave_pattern = self.wave_pattern
+        if wave_pattern:
             mask = self.apply_wave(mask)
 
         # random flip mask vertically or horizontally
-        if self.noise_type not in [5, 6, 7, 8]:
+        if self.noise_side == "random":
             if random.choice([True, False]):
                 mask = cv2.flip(mask, 0)
             if random.choice([True, False]):
                 mask = cv2.flip(mask, 1)
 
         # add dotted noise effect to mask (unsmoothen)
-        if not self.blur_noise:
+        if not blur_noise:
             noise_mask = np.random.random((ysize, xsize)) * 225
             mask[mask > noise_mask] = 255
         noise_img = mask
@@ -246,13 +281,39 @@ class BadPhotoCopy(Augmentation):
         result = cv2.multiply(noise_img, image, scale=1 / 255)
 
         # merge sobel mask and noise mask to image
-        image_original = image.copy()
-        image_copy = image.copy()
-        result_new = result.astype("int") + image_sobel.astype("int")
-        image_original[image_original > result_new] = 0
-        result_new[result_new > 255] = 0
-        result_new[result_new > image_copy] = 0
-        result = image_original + result_new
+        if self.edge_effect == -1:
+            edge_effect = random.choice((0, 1))
+        else:
+            edge_effect = self.edge_effect
+        if edge_effect:
+
+            # add random noise range from 0-128
+            add_edge_noise_fn = lambda x, y: random.randint(0, 128) if (y == 255 and random.random() < 0.70) else x
+            add_edge_noise = np.vectorize(add_edge_noise_fn)
+
+            # get edge mask
+            image_sobel = sobel(image)
+            image_sobel = cv2.GaussianBlur(image_sobel, (3, 3), 0)
+            image_sobel[:, :][image_sobel[:, :] % 255 != 0] = 255
+            image_sobel = cv2.dilate(image_sobel, (15, 15), iterations=5)
+            image_sobel_sobel = sobel(image_sobel)
+            image_sobel_sobel = cv2.dilate(image_sobel_sobel, (5, 5), iterations=2)
+            image_sobel = add_edge_noise(image_sobel, image_sobel_sobel)
+            image_sobel = cv2.GaussianBlur(image_sobel, (5, 5), 0)
+
+            image_original = image.copy()
+            image_copy = image.copy()
+
+            # apply edge
+            result_new = result.astype("int") + image_sobel.astype("int")
+            image_original[image_original > result_new] = 0
+            result_new[result_new > 255] = 0
+            result_new[result_new >= image_copy] = 0
+            result = image_original + result_new
+
+        # convert back to original image shape
+        if image_shape_length > 2:
+            result = cv2.cvtColor(result.astype("uint8"), cv2.COLOR_GRAY2BGR)
 
         return result.astype("uint8")
 
